class TwiVKStates:

    INITIALIZING = "initializing"
    DISABLED = "disabled"
    STARTING = "starting"
    SHUTTING_DOWN = "shutting_down"
    READY = "ready"
    ERROR = "error"

class TwiAPIStates(TwiVKStates):
    pass